#!/usr/bin/env python

# Copyright (c) 2010-2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import copy
import sys
import volfilter

def copy_stack (old_xl,suffix,recursive=False):
	if recursive:
		new_name = old_xl.name + "-" + suffix
	else:
		new_name = suffix
	new_xl = volfilter.Translator(new_name)
	new_xl.type = old_xl.type
	# The results with normal assignment here are . . . amusing.
	new_xl.opts = copy.deepcopy(old_xl.opts)
	for sv in old_xl.subvols:
		new_xl.subvols.append(copy_stack(sv,suffix,True))
	# Patch up the path at the bottom.
	if new_xl.type == "storage/posix":
		new_xl.opts["directory"] += ("/" + suffix)
	return new_xl

if len(sys.argv) < 3:
	print >> sys.stderr, "Usage: %s orig_volfile tenant..." % sys.argv[0]
	sys.exit(1)

graph, last = volfilter.load(sys.argv[1])

# Add the pre-authentication subvolume.
junk_sv = last.subvols[0]
while junk_sv.type != "storage/posix":
	junk_sv = junk_sv.subvols[0]
subvols = [copy_stack(junk_sv,"junk")]

# Add the per-tenant subvolumes.
for suffix in sys.argv[2:]:
	subvols.append(copy_stack(last.subvols[0],suffix))

# One cloud to bring them all...
my_xl = volfilter.Translator(last.subvols[0].name)
my_xl.type = "cluster/cloud"
my_xl.subvols = subvols

# ...and splice it in.
last.subvols = [my_xl]

volfilter.generate(graph,last)
