#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

from bottle import route, post, run, view, debug, request, response, \
		   TEMPLATE_PATH

import atexit
import optparse
import os
import sys
import socket
import string
import time

import hfs_paths
import hfs_utils
import volmap
import hfs_utils
import hfs_add_node
import hfs_add_directory
import hfs_add_volume
import hfs_delete_volume
import hfs_start_volume
import hfs_stop_volume
import hfs_add_tenant
import hfs_delete_tenant
import hfs_enable_tenant
import hfs_update_cert

@route("/")
@route("/cfg")
@route("/cfgmain")
@view("cfgmain.html")
def cfg_main():
	return dict()

@route("/cluster")
@view("cluster.html")
def show_cluster():
	# TBD: handle glusterd presence/startup check sanely
	hfs_utils.run_cmd("chkconfig","--add glusterd")
	hfs_utils.run_cmd("chkconfig","glusterd on")
	hfs_utils.run_cmd("service","glusterd start")
	node_list = string.join(hfs_utils.get_members(),"<br />")
	cert_path = os.path.join(hfs_paths.info_dir,"server.pem")
	if os.path.isfile(cert_path):
		cert_link='<a href="/cluster/certificate">get certificate</a>'
		cert_para="<p>%s</p><br />" % cert_link
	else:
		cert_para=""
	return dict(node_list=node_list,cert_para=cert_para)

@route("/cluster/certificate")
def show_cert():
	response.content_type = "text/plain"
	cert_path = os.path.join(hfs_paths.info_dir,"server.pem")
	return open(cert_path,"r")

@post("/cluster/add_node")
def add_node():
	return hfs_add_node.run_www()

# TBD: implement remove_node

@route("/volumes")
@view("volumes.html")
def show_volumes():
	brick_list = hfs_utils.get_bricks()
	mount_list = hfs_utils.get_mounts(brick_list)
	node_list = hfs_utils.get_members()
	# TBD: allow adding arbitrary directories instead of just mountpoints
	return dict(bricks=brick_list,mounts=mount_list,nodes=node_list)

@post("/volumes/add_directory")
def add_directory ():
	return hfs_add_directory.run_www()

@post("/volumes/add_dir_local")
def add_dir_local ():
	path = request.forms.get("path")
	return hfs_add_directory.add_local(path)

@post("/volumes/add_volume")
def add_volume():
	return hfs_add_volume.run_www()

@post("/volumes/add_vol_local")
def add_vol_local ():
	vname = request.forms.get("vname")
	use_ssl = request.forms.get("use_ssl")
	return hfs_add_volume.add_local(vname,use_ssl)

@route("volumes/:vol_name/remove")
def rm_volume (vol_name):
	return hfs_delete_volume.run_www(vol_name)

@route("/volumes/:vol_name/rm_local")
def rm_vol_local (vol_name):
	return hfs_delete_volume.rm_local(vol_name)

@route("/volumes/:vol_name/start")
def start_volume(vol_name):
	return hfs_start_volume.run_www(vol_name)

@route("/volumes/:vol_name/start_local")
def start_local (vol_name):
	return hfs_start_volume.start_local(vol_name)

@route("/volumes/:vol_name/stop")
def stop_volume(vol_name):
	return hfs_stop_volume.run_www(vol_name)

@route("/volumes/:vol_name/stop_local")
def stop_local (vol_name):
	return hfs_stop_volume.stop_local(vol_name)

@route("/volumes/:vol_name/tenants")
@view("volume_tenants.html")
def volume_tenants (vol_name):
	db_obj = hfs_utils.open_db()
	tenants = db_obj["vt_"+vol_name].split(",")
	# Fix bug #18 (exception when tenant list is empty)
	if tenants == [""]:
		tenants = []
	tenants.sort()
	return dict(vol_name=vol_name,tenants=tenants,db=db_obj)

# Used by mount.hekafs
@route("/:vol_name/fetch")
def fetch_client_vf(vol_name):
	vf_path = "%s/vols/%s/%s-fuse.vol" % (hfs_paths.gfs_dir, vol_name, vol_name)
	return open(vf_path,"r")

# Used by mount.hekafs
@route("/:vol_name/map")
def map_paths(vol_name):
	return volmap.vol_map(vol_name)

@route("/tenants")
@view("tenants.html")
def show_tenants ():
	db_obj = hfs_utils.open_db()
	return dict(tenants=db_obj)

@post("/tenants/add")
def add_tenant():
	return hfs_add_tenant.run_www()

@post("/tenants/add_local")
def add_tenant_local ():
	tn_name = request.forms.get("tn_name")
	tn_pw = request.forms.get("tn_pw")
	tn_uid_low = request.forms.get("tn_uid_low")
	tn_uid_high = request.forms.get("tn_uid_high")
	tn_gid_low = request.forms.get("tn_gid_low")
	tn_gid_high = request.forms.get("tn_gid_high")
	return hfs_add_tenant.add_local(tn_name,tn_pw,tn_uid_low,tn_uid_high,tn_gid_low,tn_gid_high)

@route("/tenants/:tn_name/delete")
def delete_tenant (tn_name):
	return hfs_delete_tenant.run_www(tn_name)

@route("/tenants/:tn_name/delete_local")
def delete_tenant_local (tn_name):
	return hfs_delete_tenant.delete_local(tn_name)

@route("/tenants/:tn_name/volumes")
@view("tenant_volumes.html")
def show_tenant_volumes (tn_name):
	db_obj = hfs_utils.open_db()
	all_vols = [v[3:] for v in db_obj.keys() if v.startswith("vt_")]
	# print all_vols
	active = []
	maybe_active = db_obj["tv_"+tn_name].split(",")
	if maybe_active != [""]:
		# Filter to fix bug #19 (tenant/volume inconsistencies)
		changed = False
		for vol in maybe_active:
			if db_obj.has_key("vt_"+vol):
				if tn_name in db_obj["vt_"+vol].split(","):
					active.append(vol)
					continue
			changed = True
		if changed:
			db_obj["tv_"+tn_name] = string.join(active,",")
		active.sort()
	active.sort()
	# print active
	return dict(tn_name=tn_name,all_vols=all_vols,active=active)

@post("/tenants/:tn_name/enable")
def enable_tenant_volumes (tn_name):
	return hfs_enable_tenant.run_www(tn_name)

@post("/tenants/enable_local")
def enable_tenant_local ():
	tn_name = request.forms.get("tn_name")
	vol_list = request.forms.get("vol_list")
	return hfs_enable_tenant.enable_local(tn_name,vol_list)

@route("/tenants/:tn_name/show_cert")
@view("tenant_show_cert.html")
def tenant_show_cert (tn_name):
	cert_path = os.path.join(hfs_paths.info_dir,"client_%s.pem"%tn_name)
	if os.path.isfile(cert_path):
		cert_data = open(cert_path).read()
	else:
		cert_data = ""
	return dict(tn_name=tn_name,cert_data=cert_data)

@post("/tenants/:tn_name/update_cert")
def tenant_update_cert (tn_name):
	cert_data = request.forms.get("cert_data")
	return hfs_update_cert.run_www(tn_name,cert_data)

@post("/tenants/set_cert_local")
def tenant_update_cert_local ():
	tn_name = request.forms.get("tn_name")
	cert_data = request.forms.get("cert_data")
	return hfs_update_cert.update_local(tn_name,cert_data)

@route("/styles/:sheet")
def get_style (sheet):
	fqpath = sys.argv[0].rsplit('/', 1)
	return file("%s/styles/%s" % (fqpath[0], sheet), "r")

def daemonize (log_path):

	try: 
		pid = os.fork() 
		if pid > 0:
		    sys.exit(0)
	except OSError, e: 
		write > sys.stderr, "first fork failed (%s)" % (e.strerror)
		sys.exit(1)

	os.setsid() 

	try: 
		pid = os.fork() 
		if pid > 0:
		    sys.exit(0)
	except OSError, e: 
		write > sys.stderr, "second fork failed (%s)" % (e.strerror)
		sys.exit(1)

	sys.stdout = open(log_path,"a+",0)
	sys.stderr = open(log_path,"a+",0)

if __name__ == "__main__":
	my_usage = "%prog [options]"
	parser = optparse.OptionParser(usage=my_usage)
	parser.add_option("-f","--foreground",dest="foreground",
			  default=False,action="store_true",
			  help="run in foreground (-l/-p ignored)")
	parser.add_option("-l","--logfile",dest="logfile",
			  default="/var/log/hekafs/hekafsd",
			  help="location for log file")
	parser.add_option("-p","--pidfile",dest="pidfile",
			  default="/var/run/hekafsd.pid",
			  help="location for PID file")
	options, args = parser.parse_args()
	if len(args) < 0:
		parser.print_help()
		sys.exit(1)

	if not options.foreground:
		daemonize(options.logfile)
		atexit.register(os.remove,options.pidfile)
		open(options.pidfile,"w").write(str(os.getpid())+"\n")

	debug(True)
	#run(host='0.0.0.0',port=hfs_paths.HEKAFSD_PORT,server=CherryPyServer)
	fqpath = sys.argv[0].rsplit('/', 1)
	TEMPLATE_PATH.append("%s/views" % fqpath[0])
	run(host='0.0.0.0',port=hfs_paths.HEKAFSD_PORT)
