#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib
import urllib2

from bottle import request, template
import hfs_utils
import hfs_paths

c_range_re = re.compile("\[(?P<first>.)-(?P<last>.)\\]")
n_range_re = re.compile("\[(?P<first>[1-9][0-9]*)-(?P<last>[1-9][0-9]*)\\]")
a_range_re = re.compile("\{(?P<stuff>[^}]+)\}")

def expand (path):
	old_paths = [path]
	# Keep going until there are no more expansions.
	while True:
		new_paths = []
		n_subs = 0
		for p in old_paths:
			m = c_range_re.search(p)
			if m:
				pfx = p[:m.start()]
				sfx = p[m.end():]
				first = m.group("first")
				last = m.group("last")
				cur = first
				while True:
					munged = pfx + cur + sfx
					new_paths.append(munged)
					if cur == last:
						break
					cur = chr(ord(cur)+1)
				n_subs += 1
				continue
			m = n_range_re.search(p)
			if m:
				pfx = p[:m.start()]
				sfx = p[m.end():]
				first = int(m.group("first"))
				last = int(m.group("last"))
				cur = first
				while True:
					munged = "%s%d%s" % (pfx, cur, sfx)
					new_paths.append(munged)
					if cur == last:
						break
					cur += 1
				n_subs += 1
				continue
			m = a_range_re.search(p)
			if m:
				pfx = p[:m.start()]
				sfx = p[m.end():]
				for alt in m.group("stuff").split(","):
					munged = pfx + alt + sfx
					new_paths.append(munged)
				n_subs += 1
				continue
			new_paths.append(p)
		if not n_subs:
			return new_paths
		old_paths = new_paths
		# Go back and do it again.

def add_local (path):
	# TBD: all sorts of input-validity checking
	db_obj = hfs_utils.open_db()
	# Get the current list of extra paths.
	paths_by_node = {}
	for node in [k for k in db_obj.keys() if k.startswith("xp_")]:
		xp_list = db_obj.get(node).split(",")
		if xp_list == [""]:
			xp_list = []
		paths_by_node[node[3:]] = set(xp_list)
	# Add the user's paths.
	members = hfs_utils.get_members()
	for p in expand(path):
		parts = string.split(p,":")
		if len(parts) != 2:
			return "add_local(%s) rejected %s on %s (no node)" %(
				path, p, socket.gethostname())
		node, dir = parts
		if node not in members:
			return "add_local(%s) rejected %s on %s (bad node)" %(
				path, p, socket.gethostname())
		if paths_by_node.has_key(node):
			paths_by_node[node].add(dir)
		else:
			paths_by_node[node] = set([dir])
	# Store back the results.
	for key, value in paths_by_node.iteritems():
		db_obj["xp_"+key] = string.join(value,",")
	return "add_local(%s) OK on %s" % (path, socket.gethostname())

def run_common (path):
	node_list = hfs_utils.get_members()
	blob = []
	for node in node_list:
		scratch = [node, []]
		if socket.gethostbyname(node) in hfs_utils.local_addrs:
			url_obj = [add_local(path)]
		else:
			url = "http://%s:%d/volumes/add_dir_local" % (
				node, hfs_paths.HEKAFSD_PORT)
			data = urllib.urlencode([("path",path)])
			url_obj = urllib2.urlopen(url,data=data)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www ():
	path = request.forms.get("path")
	blob = run_common(path)
	return template("add_dir_done.html",path=path,blob=blob);

if __name__ == "__main__":
	my_usage = "%prog server:path"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 1:
		parser.print_help()
		sys.exit(1)
	path = args[0]
	blob = run_common(path)
	print "Directory %s added." % path
	hfs_utils.print_blob(blob)

