#!/usr/bin/env python 

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import fileinput
import optparse
import re
import socket
import sys
import hfs_utils

from bottle import request, template

def run_common (node_name):
	host_name = socket.gethostname()

	# derive the "real" node addr, e.g. if the user enters, e.g.,
	# 192.168.122.55 and that is this IP for 'this' node then when we're
	# done can_node_addr will either be 192.168.122.55 or 127.0.0.1.
	# similarly if the user enters, e.g., <principalnode>.foo.bar.baz.com,
	# the result will be the same
	host_addr = socket.gethostbyname(host_name)
	node_addr = socket.gethostbyname(node_name)
	can_node_name = socket.gethostbyaddr(node_addr)
	can_node_addr = socket.gethostbyname(can_node_name[0])


	# now we can do the right thing if we're on the principal node or not
	if node_addr == host_addr or "127.0.0.1" == can_node_addr :
		hfs_utils.run_cmd("/sbin/chkconfig", "--add glusterd")
		hfs_utils.run_cmd("/sbin/chkconfig", "glusterd on")
		hfs_utils.run_cmd("/sbin/service", "glusterd start")
	else :
		# Remote actions.
		hfs_utils.run_cmd("/sbin/chkconfig", "--add glusterd", can_node_addr)
		hfs_utils.run_cmd("/sbin/chkconfig", "glusterd on", can_node_addr)
		hfs_utils.run_cmd("/sbin/service", "glusterd start", can_node_addr)
		# Local actions.
		hfs_utils.run_cmd("/usr/sbin/gluster", "peer probe %s" % node_name)

	return template("add_node_done.html",node_name=node_name)

def run_www ():
	node_name = request.forms.get("NODENAME")
	return run_common(node_name)

if __name__ == "__main__":
	my_usage = "%prog {node_name|node_addr}"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 1:
		parser.print_help()
		sys.exit(1)
	name = args[0]
	run_common(name)
	print "Node %s added." % name
