#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib
import urllib2

from bottle import request, template
import hfs_utils
import hfs_paths

def add_local (tn_name, tn_pw, tn_uid_low, tn_uid_high, tn_gid_low, tn_gid_high):
	# TBD: all sorts of input-validity checking
	db_obj = hfs_utils.open_db()
	db_obj["tp_"+tn_name] = tn_pw
	db_obj["tul_"+tn_name] = tn_uid_low
	db_obj["tuh_"+tn_name] = tn_uid_high
	db_obj["tgl_"+tn_name] = tn_gid_low
	db_obj["tgh_"+tn_name] = tn_gid_high
	db_obj["tv_"+tn_name] = ""
	return "add_local(%s) OK on %s" % (tn_name, socket.gethostname())

def run_common (tn_name, tn_pw, tn_uid_low, tn_uid_high, tn_gid_low, tn_gid_high):
	node_list = hfs_utils.get_members()
	blob = []
	for node in node_list:
		scratch = [node, []]
		if node in hfs_utils.local_addrs:
			url_obj = [add_local(tn_name,tn_pw,tn_uid_low,tn_uid_high,tn_gid_low,tn_gid_high)]
		else:
			url = "http://%s:%d/tenants/add_local" % (
				node, hfs_paths.HEKAFSD_PORT)
			data = urllib.urlencode([("tn_name",tn_name),
						 ("tn_pw",tn_pw),
						 ("tn_uid_low",tn_uid_low),
						 ("tn_uid_high",tn_uid_high),
						 ("tn_gid_low",tn_gid_low),
						 ("tn_gid_high",tn_gid_high)])
			url_obj = urllib2.urlopen(url,data=data)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www ():
	tn_name = request.forms.get("tn_name")
	tn_pw = request.forms.get("tn_pw")
	tn_uid_low = request.forms.get("tn_uid_low")
	tn_uid_high = request.forms.get("tn_uid_high")
	tn_gid_low = request.forms.get("tn_gid_low")
	tn_gid_high = request.forms.get("tn_gid_high")
	blob = run_common(tn_name,tn_pw,tn_uid_low,tn_uid_high,tn_gid_low,tn_gid_high)
	return template("tn_act_done.html",name=tn_name,action="added",
			blob=blob)

if __name__ == "__main__":
	my_usage = "%prog name pw uid_lo uid_hi gid_lo gid_hi"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 6:
		parser.print_help()
		sys.exit(1)
	tn_name = args[0]
	tn_pw = args[1]
	tn_uid_low = args[2]
	tn_uid_high = args[3]
	tn_gid_low = args[4]
	tn_gid_high = args[5]
	blob = run_common(tn_name,tn_pw,
		tn_uid_low,tn_uid_high,tn_gid_low,tn_gid_high)
	print "Tenant %s added." % tn_name
	hfs_utils.print_blob(blob)
