#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib
import urllib2

from bottle import request, template
import hfs_paths
import hfs_utils

def add_local (vname, use_ssl):
	db_obj = hfs_utils.open_db()
	db_obj["vt_"+vname] = ""
	db_obj["vs_"+vname] = use_ssl
	return "add_local(%s,%s) OK on %s" % (vname, use_ssl,
					      socket.gethostname())

def run_common (vname, vtype, vcount, use_ssl, bricks):
	# TBD: all sorts of input-validity checking
	# TBD: construct sane brick list
	cmd = "volume create %s" % vname
	if vtype != "plain":
		cmd = "%s %s %s" % (cmd, vtype, vcount)
	cmd = "%s %s" % (cmd, string.join(bricks))
	kid = hfs_utils.run_cmd("gluster",cmd)
	kid.communicate("y\n")
	sts = kid.wait()
	if sts:
		return [["gluster",["command failed with %d"%sts]]]
	blob = [] 
	for node in hfs_utils.get_nodes_for_vol(vname):
		scratch = [node,[]]
		if socket.gethostbyname(node) in hfs_utils.local_addrs:
			url_obj = [add_local(vname,use_ssl)]
		else:
			url = "http://%s:%d/volumes/add_vol_local" % (
				node, hfs_paths.HEKAFSD_PORT)
			data = urllib.urlencode([("vname",vname),
						 ("use_ssl",use_ssl)])
			url_obj = urllib2.urlopen(url,data=data)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www ():
	volume_id = request.forms.get("VOLUMEID")
	volume_type = request.forms.get("TYPE")
	replica_or_stripe_count = request.forms.get("COUNT")
	use_ssl = request.forms.get("SSL")
	if not use_ssl:
		use_ssl = ""
	brick_list = []
	for prop in request.forms.iterkeys():
		if prop.startswith("VOLUME_"):
			brick_list.append(prop[7:])
	blob = run_common(volume_id,volume_type,replica_or_stripe_count,use_ssl,
			  brick_list)
	return template("add_vol_done.html",name=volume_id,blob=blob);

if __name__ == "__main__":
	my_usage = "%prog [options] name brick[...]"
	parser = optparse.OptionParser(usage=my_usage)
	parser.add_option("-r","--replica",dest="replicate",default="",
		help="replica count")
	parser.add_option("-s","--stripe",dest="stripe",default="",
		help="stripe count")
	parser.add_option("-S","--ssl",dest="use_ssl",default=False,
		action="store_true",help="use SSL")
	options, args = parser.parse_args()
	if len(args) < 2:
		parser.print_help()
		sys.exit(1)
	if options.replicate:
		if options.stripe:
			print "Replication and striping are mutually exclusive"
			sys.exit(1)
		volume_type = "replica"
		replica_or_stripe_count = options.replicate
	elif options.stripe:
		volume_type = "stripe"
		replica_or_stripe_count = options.stripe
	else:
		volume_type = "plain"
		replica_or_stripe_count = ""
	if options.use_ssl:
		use_ssl = "on"
	else:
		use_ssl = ""
	volume_id = args[0]
	brick_list = args[1:]
	blob = run_common(volume_id,volume_type,replica_or_stripe_count,use_ssl,
			  brick_list)
	print "Volume %s added." % volume_id
	hfs_utils.print_blob(blob)
