#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib2

from bottle import request, template
import hfs_utils
import hfs_paths

def delete_local (tn_name):
	# TBD: all sorts of input-validity checking
	db_obj = hfs_utils.open_db()
	del db_obj["tp_"+tn_name]
	del db_obj["tul_"+tn_name]
	del db_obj["tuh_"+tn_name]
	del db_obj["tgl_"+tn_name]
	del db_obj["tgh_"+tn_name]
	del db_obj["tv_"+tn_name]
	return "delete_local(%s) OK on %s" % (tn_name, socket.gethostname())

def run_common (tn_name):
	node_list = hfs_utils.get_members()
	blob = []
	for node in node_list:
		scratch = [node, []]
		if socket.gethostbyname(node) in hfs_utils.local_addrs:
			url_obj = [delete_local(tn_name)]
		else:
			url = "http://%s:%d/tenants/%s/delete_local" % (
				node, hfs_paths.HEKAFSD_PORT, tn_name)
			url_obj = urllib2.urlopen(url)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www (tn_name):
	blob = run_common(tn_name)
	return template("tn_act_done.html",name=tn_name,action="deleted",
			blob=blob);

if __name__ == "__main__":
	my_usage = "%prog tenant"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 1:
		parser.print_help()
		sys.exit(1)
	tn_name = args[0]
	blob = run_common(tn_name)
	print "Tenant %s deleted." % tn_name
	hfs_utils.print_blob(blob)
