#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib
import urllib2

from bottle import request, template
import hfs_paths
import hfs_utils

def rm_local (vname):
	db_obj = hfs_utils.open_db()
	# Can't actually delete, but clearing is functionally equivalent.
	db_obj["vt_"+vname] = ""
	return "rm_local(%s) OK on %s" % (vname, socket.gethostname())

def run_common (vname):
	# TBD: all sorts of input-validity checking
	nodes_for_vol = hfs_utils.get_nodes_for_vol(vname)
	cmd = "volume delete %s" % vname
	kid = hfs_utils.run_cmd("gluster",cmd)
	blob = [["gluster",kid.communicate("y\n")]]
	sts = kid.wait()
	if sts:
		return template("rm_vol_done.html", name=vname, blob=blob)
	for node in nodes_for_vol:
		scratch = [node,[]]
		if socket.gethostbyname(node) in hfs_utils.local_addrs:
			url_obj = [rm_local(vname)]
		else:
			url = "http://%s:%d/volumes/%s/rm_local" % (
				node, hfs_paths.HEKAFSD_PORT, vname)
			url_obj = urllib2.urlopen(url)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www (vname):
	blob = run_common(vname)
	return template("rm_vol_done.html",name=vname,blob=blob);

if __name__ == "__main__":
	my_usage = "%prog volume"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 1:
		parser.print_help()
		sys.exit(1)
	name = args[0]
	blob = run_common(name)
	print "Volume %s removed." % name
	hfs_utils.print_blob(blob)

