#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib
import urllib2

from bottle import request, template
import hfs_paths
import hfs_utils

def enable_local (tn_name, vol_list):

	# TBD: all sorts of input-validity checking
	db_obj = hfs_utils.open_db()
	db_obj["tv_"+tn_name] = vol_list
	for vol in [v[3:] for v in db_obj.keys() if v.startswith("vt_")]:
		vt_list = db_obj["vt_"+vol].split(",")
		if vt_list == ['']:
			# This is one of the stupider split() behaviors.
			vt_list = []
		vt_list = set(vt_list)
		if vol in vol_list.split(","):
			print "enabling %s" % vol
			vt_list.add(tn_name)
		else:
			print "disabling %s" % vol
			vt_list.discard(tn_name)
		db_obj["vt_"+vol] = string.join(vt_list,",")
	return "Volumes enabled for %s on %s" % (tn_name, socket.gethostname())

def run_common (tn_name, vol_list):
	vol_list = string.join(vol_list,",")
	node_list = hfs_utils.get_members()
	blob = []
	for node in node_list:
		scratch = [node, []]
		if socket.gethostbyname(node) in hfs_utils.local_addrs:
			url_obj = [enable_local(tn_name,vol_list)]
		else:
			url = "http://%s:%d/tenants/enable_local" % (
				node, hfs_paths.HEKAFSD_PORT)
			data = urllib.urlencode([("tn_name",tn_name),
					      ("vol_list",vol_list)])
			url_obj = urllib2.urlopen(url,data=data)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www (tn_name):
	vol_list = []
	for prop in request.forms.iterkeys():
		print prop
		if prop.startswith("VOLUME_"):
			vol_list.append(prop[7:])
	blob = run_common(tn_name,vol_list)
	return template("tn_act_done.html",name=tn_name,action="enabled",
			blob=blob)

if __name__ == "__main__":
	my_usage = "%prog tenant volume[...]"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) < 2:
		parser.print_help()
		sys.exit(1)
	name = args[0]
	vols = args[1:]
	blob = run_common(name,vols)
	print "Volumes %s enabled for %s." % (string.join(vols,","), name)
	hfs_utils.print_blob(blob)
	
