#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import fileinput
import glob
import os
import socket
import subprocess
import sys
import urllib2

from bottle import template

import hfs_paths
import hfs_utils

def kill_daemons (vol_name):
	myglob = os.path.join(hfs_paths.pid_dir,"%s.*"%vol_name)
	for f in glob.iglob(myglob):
		fp = open(f,"r")
		pid = fp.read()[:-1]
		print "killing %s" % pid
		hfs_utils.run_cmd("kill",pid).wait()
	return 0

def recycle_port (path):
	print "recycling %s" % path
	port_num = os.path.basename(path)
	os.unlink(path)
	fp = open("%s/%s" % (hfs_paths.idle_subdir, port_num), "w")
	fp.close()

def stop_local (vol_name):
	retcode = kill_daemons(vol_name)
	for symlink in glob.glob(hfs_paths.used_subdir + "/*"):
		vol_link = os.readlink(symlink)
		vol = os.path.basename(vol_link)
		tokens = vol.split(".")
		if vol_name == tokens[0]:
			recycle_port(symlink)
	return "stop_local(%s) returned %d on %s\n" % (
		vol_name, retcode, socket.gethostname())


def run_common (vol_name):
	node_list = hfs_utils.get_nodes_for_vol(vol_name)
	blob = []
	for node in node_list:
		scratch = [node, []]
		if socket.gethostbyname(node) in hfs_utils.local_addrs:
			url_obj = [stop_local(vol_name)]
		else:
			url = "http://%s:%d/volumes/%s/stop_local" % (
				node, hfs_paths.HEKAFSD_PORT, vol_name)
			url_obj = urllib2.urlopen(url)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www (vol_name):
	blob = run_common(vol_name)
	return template("stop_done.html",name=vol_name,blob=blob)
	
if __name__ == "__main__":
	my_usage = "%prog volume"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 1:
		parser.print_help()
		sys.exit(1)
	name = args[0]
	blob = run_common(name)
	print "Volume %s stopped." % name
	hfs_utils.print_blob(blob)
	run_common(sys.argv[1])
