#!/usr/bin/env python

# Copyright (c) 2011 Red Hat, Inc.
#
# This file is part of HekaFS.
#
# HekaFS is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License, version 3, as published by the Free
# Software Foundation.
#
# HekaFS is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License * along
# with HekaFS.  If not, see <http://www.gnu.org/licenses/>.

import optparse
import os
import re
import socket
import string
import sys
import urllib
import urllib2

from bottle import request, template
import hfs_utils
import hfs_paths

def update_local (tn_name, cert_data):
	cert_path = os.path.join(hfs_paths.info_dir,"client_%s.pem"%tn_name)
	cert_file = open(cert_path,"w+")
	cert_file.write(cert_data)
	cert_file.flush()
	return "update_local(%s) OK on %s" % (tn_name, socket.gethostname())

def run_common (tn_name, cert_data):
	node_list = hfs_utils.get_members()
	blob = []
	for node in node_list:
		scratch = [node, []]
		if node in hfs_utils.local_addrs:
			url_obj = [update_local(tn_name,cert_data)]
		else:
			url = "http://%s:%d/tenants/set_cert_local" % (
				node, hfs_paths.HEKAFSD_PORT)
			data = urllib.urlencode([("tn_name",tn_name),
						 ("cert_data",cert_data)])
			url_obj = urllib2.urlopen(url,data=data)
		for line in url_obj:
			scratch[1].append(line)
		blob.append(scratch)
	return blob

def run_www (tn_name, cert_data):
	blob = run_common(tn_name,cert_data)
	return template("tn_act_done.html",name=tn_name,action="updated",
			blob=blob)

if __name__ == "__main__":
	my_usage = "%prog tenant cert-path"
	parser = optparse.OptionParser(usage=my_usage)
	options, args = parser.parse_args()
	if len(args) != 2:
		parser.print_help()
		sys.exit(1)
	tn_name = args[0]
	cert_data = open(args[1],"r").read()
	blob = run_common(tn_name,cert_data)
	print "Tenant %s updated." % tn_name
	hfs_utils.print_blob(blob)
